package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.constants.LogAndUpdateTipsTypeEnum;
import com.gic.cloud.constants.UpdateTypeEnum;
import com.gic.cloud.dto.IndexDTO;
import com.gic.cloud.entity.TabIndex;
import com.gic.cloud.entity.TabIndexDesc;
import com.gic.cloud.entity.TabIndexGroup;
import com.gic.cloud.qo.IndexQo;
import com.gic.cloud.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-08 10:19
 */
@Service("indexApiService")
public class IndexApiServiceImpl implements IndexApiService {

    private static final String HAOBAN_NAME = "haoban";

    @Autowired
    private IndexService indexService;
    @Autowired
    private IndexLogService indexLogService;
    @Autowired
    private IndexModuleRelService indexModuleRelService;
    @Autowired
    private UpdateTipService updateTipService;
    @Autowired
    private IndexGroupService indexGroupService;
    @Autowired
    private IndexDescService indexDescService;

    @Override
    public ServiceResponse<Void> saveIndex(IndexDTO indexDTO) {
        // 校验参数
        if (indexService.isNameRepeat(indexDTO.getName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        if (indexService.isCodeRepeat(indexDTO.getCode(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "编码重复");
        }
        // 保存实体
        Integer indexId = indexService.save(indexDTO);
        indexDTO.setIndexId(indexId);
        // 保存关联表
        indexModuleRelService.save(indexDTO.getIndexId(), indexDTO.getRelModuleIdList(), 1);
        // 写日志
        indexLogService.saveIndexInsertLog(indexDTO);

        // 红点提示
        updateTipService.insertBySaveBusiness(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateIndex(IndexDTO indexDTO, Integer updateType, String reason) {
        // 校验参数
        if (indexService.isNameRepeat(indexDTO.getName(), indexDTO.getIndexId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        if (indexService.isCodeRepeat(indexDTO.getCode(), indexDTO.getIndexId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "编码重复");
        }
        // 保存实体
        indexService.update(indexDTO);
        // 保存关联表
        indexModuleRelService.save(indexDTO.getIndexId(), indexDTO.getRelModuleIdList(), 1);

        if (UpdateTypeEnum.UPDATE_PUBLISH.getCode().equals(updateType)) {
            // 写日志
            indexLogService.saveIndexUpdateLog(indexDTO, reason);
            // 红点提示
            updateTipService.publish(indexDTO.getIndexId(), LogAndUpdateTipsTypeEnum.INDEX.getCode());
        }


        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<IndexDTO> getByIndexId(Integer indexId) {
        TabIndex index = this.indexService.getByIndexId(indexId);
        IndexDTO indexDTO = EntityUtil.changeEntityByJSON(IndexDTO.class, index);
        List<String> moduleIdList = this.indexModuleRelService.listModuleIdByBusinessId(indexDTO.getIndexId(), LogAndUpdateTipsTypeEnum.INDEX.getCode());
        indexDTO.setRelModuleIdList(moduleIdList);
        if (indexDTO.getIndexGroupId() != null) {
            TabIndexGroup indexGroup = this.indexGroupService.getById(indexDTO.getIndexGroupId());
            if (indexGroup != null) {
                indexDTO.setIndexGroupName(indexGroup.getIndexGroupName());
            }
        }
        if (StringUtils.isNotBlank(indexDTO.getRelTopLevel())) {
            List<Integer> idsList = Stream.of(indexDTO.getRelTopLevel().split(GlobalInfo.FLAG_COMMA)).map(Integer::valueOf).collect(Collectors.toList());
            IndexQo indexQo = new IndexQo();
            indexQo.setIndexIdList(idsList);
            List<TabIndex> indexList = this.indexService.listAllIndex(indexQo);
            String repTopLevelName = Optional.ofNullable(indexList).orElse(Collections.emptyList())
                    .stream().map(TabIndex::getName).collect(Collectors.joining(","));
            indexDTO.setRelTopLevelName(repTopLevelName);
        }
        return EnterpriseServiceResponse.success(indexDTO);
    }

    @Override
    public ServiceResponse<Page<IndexDTO>> listIndex(IndexQo indexQo) {
        com.github.pagehelper.Page<TabIndex> indexList = indexService.listIndex(indexQo);
        Page<IndexDTO> indexDtoPage = PageHelperUtils.changePageHelperToCurrentPage(indexList, IndexDTO.class);
        if (CollectionUtils.isNotEmpty(indexList)) {
            List<Integer> indexIdList = indexList.stream().map(TabIndex::getIndexId).collect(Collectors.toList());
            Map<Integer, List<String>> moduleIdByBusinessIdMap = indexModuleRelService.groupModuleIdByBusinessIds(indexIdList, LogAndUpdateTipsTypeEnum.INDEX.getCode());
            List<IndexDTO> indexDTOList = indexDtoPage.getResult();
            HashMap<Integer, String> cache = new HashMap<>(16);
            for (IndexDTO indexDTO : indexDTOList) {
                indexDTO.setRelModuleIdList(moduleIdByBusinessIdMap.get(indexDTO.getIndexId()));
                Integer indexGroupId = indexDTO.getIndexGroupId();
                if (indexGroupId != null) {
                    indexDTO.setIndexGroupName(getIndexGroupName(cache, indexGroupId));
                }
            }
        }
        return EnterpriseServiceResponse.success(indexDtoPage);
    }

    private String getIndexGroupName(HashMap<Integer, String> cache, Integer indexGroupId) {
        String indexGroupName = cache.get(indexGroupId);
        if (StringUtils.isNotBlank(indexGroupName)) {
            return indexGroupName;
        }
        TabIndexGroup indexGroup = indexGroupService.getById(indexGroupId);
        List<Integer> indexGroupIdList = Stream.of(indexGroup.getGroupChain().split("_")).filter(StringUtils::isNotBlank).map(Integer::valueOf).collect(Collectors.toList());
        String groupName = indexGroupService.getNameByIds(indexGroupIdList);
        cache.put(indexGroupId, groupName);
        return groupName;
    }

    @Override
    public ServiceResponse<List<IndexDTO>> listAllIndex(IndexQo indexQo) {
        List<TabIndex> tabIndices = indexService.listAllIndex(indexQo);
        List<IndexDTO> dtoList = EntityUtil.changeEntityListByJSON(IndexDTO.class, tabIndices);
        return EnterpriseServiceResponse.success(dtoList);
    }

    @Override
    public ServiceResponse<Void> delete(Integer indexId) {
        TabIndex index = this.indexService.getByIndexId(indexId);
        if (index == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "指标不存在或已删除");
        }
        List<String> moduleIdList = indexModuleRelService.listModuleIdByBusinessId(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());
        if (CollectionUtils.isNotEmpty(moduleIdList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不能删除被引用的指标");
        }
        this.indexService.deleteById(indexId);
        this.indexModuleRelService.deleteByBusinessId(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> publish(List<Integer> indexIdList, Integer updateType, String reason, Integer optUserId, String optUserName) {
        for (Integer indexId : indexIdList) {
            this.updateTipService.publish(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());
            IndexDTO indexDTO = new IndexDTO();
            indexDTO.setIndexId(indexId);
            indexDTO.setOptUserId(optUserId);
            indexDTO.setOptUserName(optUserName);
            indexLogService.saveIndexUpdateLog(indexDTO, reason);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<IndexDTO> getByIndexDescCode(String indexDescCode) {
        TabIndexDesc indexDesc = indexDescService.getIndexDesc(HAOBAN_NAME, indexDescCode, null);
        if (indexDesc == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
        List<Integer> list = indexModuleRelService.listBusinessIdByModuleId(indexDesc.getIndexDescId().toString(), LogAndUpdateTipsTypeEnum.INDEX.getCode());
        if (CollectionUtils.isEmpty(list)) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
        TabIndex index = indexService.getByIndexId(list.get(0));
        if (index == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(IndexDTO.class, index));
    }
}
